﻿#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
borealis_transparent.py

A single-file PyQt5 + OdenGraphQt script that:
  - Creates a frameless, always-on-top, fully transparent overlay window.
  - Embeds an OdenGraphQt NodeGraph with a viewer that draws NOTHING 
    in drawBackground(), resulting in a completely transparent background.
  - Monkey-patches OdenGraphQt so removing nodes won't crash.
  - Includes an "Edit Mode" button to toggle node interactivity:
      * OFF => NodeGraph is click-through (no context menu, no alt-drag).
      * ON  => Right-click the transparent background, alt-drag nodes, etc.
"""

import sys
import pkgutil
import importlib
import inspect
import types

# ---------- PyQt5 Imports ----------
from PyQt5.QtWidgets import (
    QApplication, QMainWindow, QWidget, QPushButton
)
from PyQt5.QtCore import (
    Qt, QTimer
)
from PyQt5.QtGui import (
    QPainter
)

# ---------- OdenGraphQt Monkey-Patch ----------
import OdenGraphQt.base.graph as base_graph
from PyQt5 import QtWidgets  # PyQt5's QUndoStack
base_graph.QtGui.QUndoStack = QtWidgets.QUndoStack

import OdenGraphQt.base.commands as base_commands
_original_redo = base_commands.NodesRemovedCmd.redo
_original_undo = base_commands.NodesRemovedCmd.undo

def _patched_redo(self):
    try:
        _original_redo(self)
    except TypeError as e:
        if "unexpected type" in str(e) and hasattr(self, 'node'):
            node_ids = []
            if isinstance(self.node, list):
                node_ids = [getattr(n, 'id', str(n)) for n in self.node]
            else:
                node_ids = [getattr(self.node, 'id', str(self.node))]
            self.graph.nodes_deleted.emit(node_ids)
        else:
            raise

def _patched_undo(self):
    try:
        _original_undo(self)
    except TypeError as e:
        if "unexpected type" in str(e) and hasattr(self, 'node'):
            node_ids = []
            if isinstance(self.node, list):
                node_ids = [getattr(n, 'id', str(n)) for n in self.node]
            else:
                node_ids = [getattr(self.node, 'id', str(self.node))]
            self.graph.nodes_deleted.emit(node_ids)
        else:
            raise

base_commands.NodesRemovedCmd.redo = _patched_redo
base_commands.NodesRemovedCmd.undo = _patched_undo

# ---------- OdenGraphQt Viewer with NO Background ----------
from OdenGraphQt.widgets.viewer import NodeViewer

class TransparentViewer(NodeViewer):
    """
    A NodeViewer that does not paint anything in drawBackground().
    => Fully transparent background if Windows honors alpha.
    """
    def drawBackground(self, painter, rect):
        # Do not call super(), do not paint anything => 100% transparent background
        pass

# ---------- NodeGraph & Node Import Helpers ----------
from OdenGraphQt import NodeGraph, BaseNode

def import_nodes_from_folder(package_name):
    imported_nodes = []
    package = importlib.import_module(package_name)
    for loader, module_name, is_pkg in pkgutil.walk_packages(
            package.__path__, package.__name__ + "."):
        module = importlib.import_module(module_name)
        for name, obj in inspect.getmembers(module, inspect.isclass):
            if issubclass(obj, BaseNode) and obj.__module__ == module.__name__:
                imported_nodes.append(obj)
    return imported_nodes

def make_node_command(graph, node_type):
    def command():
        try:
            graph.create_node(node_type)
        except Exception as e:
            print(f"Error creating node of type {node_type}: {e}")
    return command

# ---------- Edit Mode Button ----------
class EditButton(QPushButton):
    """
    A small, frameless button that always stays on top, toggling edit mode.
    """
    def __init__(self, parent=None):
        super().__init__("Enter Edit Mode", parent)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        self.setStyleSheet("background-color: rgba(255,255,255,200); border: 1px solid black;")
        self.resize(140, 40)

# ---------- Main Overlay Window ----------
class MainWindow(QMainWindow):
    """
    A frameless, top-level overlay with:
      - Full screen geometry from primaryScreen()
      - setAttribute(Qt.WA_TranslucentBackground), etc.
      - A NodeGraph using TransparentViewer => NO painting => 100% transparent background.
      - "Edit Mode" toggles NodeGraph interactivity (OFF => pass-through).
    """
    def __init__(self):
        super().__init__()

        # (A) Full-screen overlay
        app = QApplication.instance()
        screen_geo = app.primaryScreen().geometry()
        self.setGeometry(screen_geo)

        # (B) Frameless, top-most, fully transparent
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setAttribute(Qt.WA_NoSystemBackground, True)
        self.setAttribute(Qt.WA_OpaquePaintEvent, False)
        self.setAttribute(Qt.WA_AlwaysStackOnTop, True)

        # (C) NodeGraph with TransparentViewer
        self.graph = NodeGraph(viewer=TransparentViewer())
        self.nodeGraphWidget = self.graph.widget
        self.nodeGraphWidget.setStyleSheet("background: transparent; border: none;")

        # Transparent central widget
        central = QWidget(self)
        central.setAttribute(Qt.WA_TranslucentBackground, True)
        central.setAttribute(Qt.WA_NoSystemBackground, True)
        central.setAttribute(Qt.WA_OpaquePaintEvent, False)
        self.setCentralWidget(central)

        self.nodeGraphWidget.setParent(central)
        self.nodeGraphWidget.setGeometry(central.rect())

        # (D) Edit Mode Button
        self.editButton = EditButton(self)
        self.editButton.move(10, 10)
        self.editButton.clicked.connect(self.toggleEditMode)
        self.isEditMode = False

        # (E) Import custom nodes (optional)
        try:
            custom_nodes = import_nodes_from_folder('Nodes')
            for node_class in custom_nodes:
                self.graph.register_node(node_class)

            graph_menu = self.graph.get_context_menu('graph')
            for node_class in custom_nodes:
                node_type = f"{node_class.__identifier__}.{node_class.__name__}"
                node_name = node_class.NODE_NAME
                graph_menu.add_command(
                    f"Add {node_name}",
                    make_node_command(self.graph, node_type)
                )
            graph_menu.add_command(
                "Remove Selected Node",
                lambda: [self.graph.remove_node(n) for n in self.graph.selected_nodes()]
                         if self.graph.selected_nodes() else None
            )
        except Exception as e:
            print(f"Error setting up custom nodes: {e}")

        # (F) Global update timer
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.global_update)
        self.timer.start(500)

        # Show
        self.show()

        # Start with Edit Mode OFF => NodeGraph is non-interactive
        self.nodeGraphWidget.setAttribute(Qt.WA_TransparentForMouseEvents, True)

    def toggleEditMode(self):
        """
        Toggle NodeGraph interactivity:
          - OFF => pass-through clicks (no context menu, no alt-drag).
          - ON  => alt-drag nodes, right-click transparent background for menu, etc.
        """
        self.isEditMode = not self.isEditMode
        if self.isEditMode:
            self.editButton.setText("Exit Edit Mode")
            self.nodeGraphWidget.setAttribute(Qt.WA_TransparentForMouseEvents, False)
        else:
            self.editButton.setText("Enter Edit Mode")
            self.nodeGraphWidget.setAttribute(Qt.WA_TransparentForMouseEvents, True)

    def global_update(self):
        """
        Periodically call process_input() on each node (if implemented).
        """
        for node in self.graph.all_nodes():
            if hasattr(node, "process_input"):
                try:
                    node.process_input()
                except Exception as e:
                    print(f"Error updating node {node}: {e}")

    def resizeEvent(self, event):
        """
        Keep NodeGraph sized to the entire window.
        """
        super().resizeEvent(event)
        if self.centralWidget():
            self.nodeGraphWidget.setGeometry(self.centralWidget().rect())

# ---------- Entry Point ----------
if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = MainWindow()
    window.setWindowTitle("Project Borealis - Fully Transparent Overlay")
    window.show()
    sys.exit(app.exec_())
